@@ -1,14 +1,20 @@
 package Method::Generate::Accessor;
+use strict;
+use warnings;
 
-use Moo::_strictures;
-use Moo::_Utils qw(_load_module _maybe_load_module _install_coderef);
+use Moo::_Utils qw(_maybe_load_module _install_coderef _module_name_rx);
 use Moo::Object ();
 BEGIN { our @ISA = qw(Moo::Object) }
 use Sub::Quote qw(quote_sub quoted_from_sub quotify sanitize_identifier);
 use Scalar::Util 'blessed';
 use Carp qw(croak);
-BEGIN { our @CARP_NOT = qw(Moo::_Utils) }
-use overload ();
+BEGIN {
+  our @CARP_NOT = qw(
+    Moo::_Utils
+    Moo::Object
+    Moo::Role
+  );
+}
 BEGIN {
   *_CAN_WEAKEN_READONLY = (
     "$]" < 5.008_003 or $ENV{MOO_TEST_PRE_583}
@@ -31,10 +37,7 @@ BEGIN {
   $Carp::Internal{+__PACKAGE__} = 1;
 }
 
-my $module_name_only = qr/\A$Module::Runtime::module_name_rx\z/;
-
-sub _die_overwrite
-{
+sub _die_overwrite {
   my ($pkg, $method, $type) = @_;
   croak "You cannot overwrite a locally defined method ($method) with "
     . ( $type || 'an accessor' );
@@ -73,7 +76,7 @@ sub generate_method {
     }
     $spec->{builder} = '_build_'.$name if ($spec->{builder}||0) eq 1;
     croak "Invalid builder for $into->$name - not a valid method name"
-      if $spec->{builder} !~ $module_name_only;
+      if $spec->{builder} !~ _module_name_rx;
   }
   if (($spec->{predicate}||0) eq 1) {
     $spec->{predicate} = $name =~ /^_/ ? "_has${name}" : "has_${name}";
@@ -185,24 +188,26 @@ sub generate_method {
         exists_predicates => $into, $pred, $name, $spec
       );
     } else {
+      $self->{captures} = {};
       $methods{$pred} =
         quote_sub "${into}::${pred}"
           => $self->_generate_simple_has('$_[0]', $name, $spec)."\n"
-          => {}
+          => delete $self->{captures}
           => $quote_opts
         ;
     }
   }
-  if (my $pred = $spec->{builder_sub}) {
-    _install_coderef( "${into}::$spec->{builder}" => $spec->{builder_sub} );
+  if (my $builder = delete $spec->{builder_sub}) {
+    _install_coderef( "${into}::$spec->{builder}" => $builder );
   }
   if (my $cl = $spec->{clearer}) {
     _die_overwrite($into, $cl, 'a clearer')
       if !$spec->{allow_overwrite} && defined &{"${into}::${cl}"};
+    $self->{captures} = {};
     $methods{$cl} =
       quote_sub "${into}::${cl}"
         => $self->_generate_simple_clear('$_[0]', $name, $spec)."\n"
-        => {}
+        => delete $self->{captures}
         => $quote_opts
       ;
   }
@@ -216,7 +221,6 @@ sub generate_method {
           keys %$hspec;
       } elsif (!ref($hspec)) {
         require Moo::Role;
-        _load_module $hspec;
         map [ $_ => $_ ], Moo::Role->methods_provided_by($hspec)
       } else {
         croak "You gave me a handles of ${hspec} and I have no idea why";
@@ -249,6 +253,32 @@ sub generate_method {
   \%methods;
 }
 
+sub merge_specs {
+  my ($self, @specs) = @_;
+  my $spec = shift @specs;
+  for my $old_spec (@specs) {
+    foreach my $key (keys %$old_spec) {
+      if ($key eq 'handles') {
+      }
+      elsif ($key eq 'moosify') {
+        $spec->{$key} = [
+          map { ref $_ eq 'ARRAY' ? @$_ : $_ }
+          grep defined,
+          ($old_spec->{$key}, $spec->{$key})
+        ];
+      }
+      elsif ($key eq 'builder' || $key eq 'default') {
+        $spec->{$key} = $old_spec->{$key}
+          if !(exists $spec->{builder} || exists $spec->{default});
+      }
+      elsif (!exists $spec->{$key}) {
+        $spec->{$key} = $old_spec->{$key};
+      }
+    }
+  }
+  $spec;
+}
+
 sub is_simple_attribute {
   my ($self, $name, $spec) = @_;
   # clearer doesn't have to be listed because it doesn't
@@ -460,9 +490,9 @@ sub _wrap_attr_exception {
   .$code."),\n"
   ."    1\n"
   ."  } or\n"
-  .'    $_error = ref $@ ? $@ : '.$prefix.'.$@);'."\n"
+  .'    $_error = CORE::ref $@ ? $@ : '.$prefix.'.$@);'."\n"
   .'  ($@ = $_old_error),'."\n"
-  .'  (defined $_error and die $_error);'."\n"
+  .'  (defined $_error and CORE::die $_error);'."\n"
   .($want_return ? '  $_return;'."\n" : '')
   ."}\n"
 }
@@ -548,10 +578,10 @@ sub _generate_populate_set {
     $spec->{trigger}
   ) : undef;
   if ($has_default) {
-    "($set)," . ($trigger ? "($test and $trigger)," : '')
+    "($set)," . ($trigger && $test ? "($test and $trigger)," : '') . "\n";
   }
   else {
-    "($test and ($set)" . ($trigger ? ", ($trigger)" : '') . "),";
+    "($test and ($set)" . ($trigger ? ", ($trigger)" : '') . "),\n";
   }
 }
 
@@ -610,11 +640,11 @@ sub _generate_getset {
 
 sub _generate_asserter {
   my ($self, $name, $spec) = @_;
-
+  my $name_str = quotify($name);
   "do {\n"
    ."  my \$val = ".$self->_generate_get($name, $spec).";\n"
    ."  ".$self->_generate_simple_has('$_[0]', $name, $spec)."\n"
-   ."    or Carp::croak(\"Attempted to access '${name}' but it is not set\");\n"
+   ."    or Carp::croak(q{Attempted to access '}.${name_str}.q{' but it is not set});\n"
    ."  \$val;\n"
    ."}\n";
 }
@@ -647,19 +677,27 @@ sub default_construction_string { '{}' }
 
 sub _validate_codulatable {
   my ($self, $setting, $value, $into, $appended) = @_;
-  my $invalid = "Invalid $setting '" . overload::StrVal($value)
-    . "' for $into not a coderef";
-  $invalid .= " $appended" if $appended;
 
-  unless (ref $value and (ref $value eq 'CODE' or blessed($value))) {
-    croak "$invalid or code-convertible object";
+  my $error;
+
+  if (blessed $value) {
+    local $@;
+    no warnings 'void';
+    eval { \&$value; 1 }
+      and return 1;
+    $error = "could not be converted to a coderef: $@";
+  }
+  elsif (ref $value eq 'CODE') {
+    return 1;
+  }
+  else {
+    $error = 'is not a coderef or code-convertible object';
   }
 
-  unless (eval { \&$value }) {
-    croak "$invalid and could not be converted to a coderef: $@";
-  }
-
-  1;
+  croak "Invalid $setting '"
+    . ($INC{'overload.pm'} ? overload::StrVal($value) : $value)
+    . "' for $into " . $error
+    . ($appended ? " $appended" : '');
 }
 
 1;
