@@ -1,12 +1,13 @@
 package Method::Generate::Constructor;
+use strict;
+use warnings;
 
-use Moo::_strictures;
 use Sub::Quote qw(quote_sub quotify);
 use Sub::Defer;
-use Moo::_Utils qw(_getstash _getglob);
-use Moo::_mro;
+use Moo::_Utils qw(_getstash _getglob _linear_isa);
 use Scalar::Util qw(weaken);
 use Carp qw(croak);
+use Carp::Heavy ();
 BEGIN { our @CARP_NOT = qw(Sub::Defer) }
 BEGIN {
   local $Moo::sification::disabled = 1;
@@ -18,26 +19,16 @@ sub register_attribute_specs {
   my ($self, @new_specs) = @_;
   $self->assert_constructor;
   my $specs = $self->{attribute_specs}||={};
+  my $ag = $self->accessor_generator;
   while (my ($name, $new_spec) = splice @new_specs, 0, 2) {
     if ($name =~ s/^\+//) {
       croak "has '+${name}' given but no ${name} attribute already exists"
         unless my $old_spec = $specs->{$name};
-      foreach my $key (keys %$old_spec) {
-        if (!exists $new_spec->{$key}) {
-          $new_spec->{$key} = $old_spec->{$key}
-            unless $key eq 'handles';
-        }
-        elsif ($key eq 'moosify') {
-          $new_spec->{$key} = [
-            map { ref $_ eq 'ARRAY' ? @$_ : $_ }
-              ($old_spec->{$key}, $new_spec->{$key})
-          ];
-        }
-      }
+      $ag->merge_specs($new_spec, $old_spec);
     }
     if ($new_spec->{required}
       && !(
-        $self->accessor_generator->has_default($name, $new_spec)
+        $ag->has_default($name, $new_spec)
         || !exists $new_spec->{init_arg}
         || defined $new_spec->{init_arg}
       )
@@ -84,11 +75,11 @@ sub install_delayed {
   my ($self) = @_;
   $self->assert_constructor;
   my $package = $self->{package};
-  my (undef, @isa) = @{mro::get_linear_isa($package)};
+  my (undef, @isa) = @{_linear_isa($package)};
   my $isa = join ',', @isa;
   my (undef, $from_file, $from_line) = caller(Carp::short_error_loc());
   my $constructor = defer_sub "${package}::new" => sub {
-    my (undef, @new_isa) = @{mro::get_linear_isa($package)};
+    my (undef, @new_isa) = @{_linear_isa($package)};
     if (join(',', @new_isa) ne $isa) {
       my ($expected_new) = grep { *{_getglob($_.'::new')}{CODE} } @isa;
       my ($found_new) = grep { *{_getglob($_.'::new')}{CODE} } @new_isa;
@@ -145,8 +136,8 @@ sub generate_method {
   my $into_buildargs = $into->can('BUILDARGS');
 
   my $body
-    = '    my $invoker = shift;'."\n"
-    . '    my $class = ref($invoker) ? ref($invoker) : $invoker;'."\n"
+    = '    my $invoker = CORE::shift();'."\n"
+    . '    my $class = CORE::ref($invoker) ? CORE::ref($invoker) : $invoker;'."\n"
     . $self->_handle_subconstructor($into, $name)
     . ( $into_buildargs && $into_buildargs != \&Moo::Object::BUILDARGS
       ? $self->_generate_args_via_buildargs
@@ -191,7 +182,7 @@ sub _cap_call {
 sub _generate_args_via_buildargs {
   my ($self) = @_;
   q{    my $args = $class->BUILDARGS(@_);}."\n"
-  .q{    Carp::croak("BUILDARGS did not return a hashref") unless ref($args) eq 'HASH';}
+  .q{    Carp::croak("BUILDARGS did not return a hashref") unless CORE::ref($args) eq 'HASH';}
   ."\n";
 }
 
@@ -200,7 +191,7 @@ sub _generate_args {
   my ($self) = @_;
   return <<'_EOA';
     my $args = scalar @_ == 1
-      ? ref $_[0] eq 'HASH'
+      ? CORE::ref $_[0] eq 'HASH'
         ? { %{ $_[0] } }
         : Carp::croak("Single parameters to new() must be a HASH ref"
             . " data => ". $_[0])
@@ -240,13 +231,13 @@ sub _check_required {
   my @required_init =
     map $spec->{$_}{init_arg},
       grep {
-        my %s = %{$spec->{$_}}; # ignore required if default or builder set
-        $s{required} and not($s{builder} or exists $s{default})
+        my $s = $spec->{$_}; # ignore required if default or builder set
+        $s->{required} and not($s->{builder} or exists $s->{default})
       } sort keys %$spec;
   return '' unless @required_init;
   '    if (my @missing = grep !exists $args->{$_}, '
     .join(', ', map quotify($_), @required_init).') {'."\n"
-    .q{      Carp::croak("Missing required arguments: ".join(', ', sort @missing));}."\n"
+    .q{      Carp::croak("Missing required arguments: ".CORE::join(', ', sort @missing));}."\n"
     ."    }\n";
 }
 
